{#

extra variables:
- user_name: the name of the user
- verification_code: the verification code
- site_title: the title of the site
- site_url: the URL of the site

#}

<!DOCTYPE html>
<html>

<head>
    <meta charset="UTF-8">
    <title>Login Verification Code</title>
</head>

<body>
    <h1>Login Verification Code</h1>

    <p>Dear {{ user_name }},</p>
    <p>Thank you for using our platform. To complete your login process, please enter the verification code below:</p>
    <p><strong>{{ verification_code }}</strong></p>

    <p>If you did not request this verification code, please ignore this email.</p>
    <p>Thank you.</p>

    <p>Message sent by {{ site_title }} (<a href="{{ site_url }}">{{ site_url }}</a>)</p>
</body>

</html>
